<?php
/*
 *  $Id: ApiAbstract.php 275 2011-05-01 11:26:41Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_ApiAbstract
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Api
 * @version    $Revision: 275 $
 */
abstract class PSX_ApiAbstract extends PSX_ModuleAbstract
{
	protected function getRequestParams()
	{
		$params = array();
		$params['fields']       = null;
		$params['updatedSince'] = null;
		$params['count']        = null;
		$params['filterBy']     = null;
		$params['filterOp']     = null;
		$params['filterValue']  = null;
		$params['sortBy']       = null;
		$params['sortOrder']    = null;
		$params['startIndex']   = null;


		if(isset($_GET['fields']))
		{
			$raw_fields = explode(',', $_GET['fields']);
			$fields     = array();

			foreach($raw_fields as $field)
			{
				$field = trim($field);

				if(strlen($field) > 1 && strlen($field) < 32 && ctype_alnum($field))
				{
					$fields[] = $field;
				}
			}

			if(!empty($fields))
			{
				$params['fields'] = $fields;
			}
		}

		if(isset($_GET['updatedSince']) && strlen($_GET['updatedSince']) < 22)
		{
			$params['updatedSince'] = $_GET['updatedSince'];
		}

		if(isset($_GET['count']))
		{
			$params['count'] = intval($_GET['count']);
		}

		if(isset($_GET['filterBy']) && ctype_alnum($_GET['filterBy']) && strlen($_GET['filterBy']) < 32)
		{
			$params['filterBy'] = $_GET['filterBy'];
		}

		if(isset($_GET['filterOp']) && in_array($_GET['filterOp'], array('contains', 'equals', 'startsWith', 'present')))
		{
			$params['filterOp'] = $_GET['filterOp'];
		}

		if(isset($_GET['filterValue']) && strlen($_GET['filterValue']) < 128)
		{
			$params['filterValue'] = $_GET['filterValue'];
		}

		if(isset($_GET['sortBy']) && strlen($_GET['sortBy']) < 128)
		{
			$params['sortBy'] = $_GET['sortBy'];
		}

		if(isset($_GET['sortOrder']) && in_array($_GET['sortOrder'], array('ascending', 'descending')))
		{
			$params['sortOrder'] = $_GET['sortOrder'];
		}

		if(isset($_GET['startIndex']))
		{
			$params['startIndex'] = intval($_GET['startIndex']);
		}


		return $params;
	}

	/**
	 * getRequest
	 *
	 * Returns an PSX_Data_ReaderResult object depending of the $reader
	 * string. If the string is not set the content-type of the request is
	 * used to get the best fitting reader. You can use the method import of
	 * an record to transform the request into an record
	 *
	 * @param integer $readerType
	 * @return PSX_Data_ReaderResult
	 */
	protected function getRequest($readerType = null)
	{
		if($readerType === null)
		{
			$readerType = $this->getPreferredReaderType();
		}


		$reader = $this->getReader($readerType);

		if($reader === null)
		{
			throw new PSX_Exception('Could not find fitting data reader');
		}


		// build request
		$url    = new PSX_Url($this->config->getSelf());
		$method = $_SERVER['REQUEST_METHOD'];
		$header = PSX_Config::getRequestHeader();
		$body   = PSX_Config::getRawInput();

		$request = new PSX_Http_Request($url, $method, $header, $body);


		return $reader->read($request);
	}

	/**
	 * setResponse
	 *
	 * Writes the $record with the writer $writerType or depending on the
	 * get parameter format or of the mime type of the Accept header.
	 *
	 * @param PSX_Data_Record $record
	 * @param integer $writerType
	 * @param integer $code
	 * @return void
	 */
	protected function setResponse(PSX_Data_RecordInterface $record, $writerType = null, $code = 200)
	{
		if(isset(PSX_Http::$codes[$code]))
		{
			header('HTTP/1.1 ' . $code . ' ' . PSX_Http::$codes[$code]);
		}
		else
		{
			throw new PSX_Exception('Invalid http code');
		}


		// find best writer type if not set
		if($writerType === null)
		{
			$writerType = $this->getPreferredWriterType();
		}


		$writer = $this->getWriter($writerType);

		if($writer === null)
		{
			throw new PSX_Exception('Could not find fitting data writer');
		}


		$writerResult = new PSX_Data_WriterResult($writerType, $writer);

		$this->setWriterConfig($writerResult);


		$writer->write($record);

		exit;
	}

	/**
	 * setWriterConfig
	 *
	 * You can override this method to configure the writer. Some writers
	 * require configuration i.e. the atom writer needs to know wich fields
	 * should be used for an entry.
	 *
	 * @param PSX_Data_WriterResult $result
	 * @return void
	 */
	protected function setWriterConfig(PSX_Data_WriterResult $result)
	{
	}

	private function getPreferredReaderType()
	{
		$readerType  = null;
		$contentType = PSX_Config::getRequestHeader('content-type');

		switch(true)
		{
			case (stripos($contentType, PSX_Data_Reader_Form::$mime) !== false):

				$readerType = PSX_Data_ReaderInterface::FORM;

				break;

			case (stripos($contentType, PSX_Data_Reader_Json::$mime) !== false):

				$readerType = PSX_Data_ReaderInterface::JSON;

				break;

			case (stripos($contentType, PSX_Data_Reader_Xml::$mime) !== false):

				$readerType = PSX_Data_ReaderInterface::XML;

				break;

			default:

				throw new PSX_Data_Exception('Invalid content type');

				break;
		}

		return $readerType;
	}

	private function getReader($readerType)
	{
		$reader = null;

		switch($readerType)
		{
			case PSX_Data_ReaderInterface::DOM:

				$reader = new PSX_Data_Reader_Dom();

				break;

			case PSX_Data_ReaderInterface::FORM:

				$reader = new PSX_Data_Reader_Form();

				break;

			case PSX_Data_ReaderInterface::JSON:

				$reader = new PSX_Data_Reader_Json();

				break;

			case PSX_Data_ReaderInterface::RAW:

				$reader = new PSX_Data_Reader_Raw();

				break;

			case PSX_Data_ReaderInterface::XML:

				$reader = new PSX_Data_Reader_Xml();

				break;
		}

		return $reader;
	}

	private function getPreferredWriterType()
	{
		$formats = array(

			'atom' => PSX_Data_Writer_Atom::$mime,
			'form' => PSX_Data_Writer_Form::$mime,
			'json' => PSX_Data_Writer_Json::$mime,
			'rss'  => PSX_Data_Writer_Rss::$mime,
			'xml'  => PSX_Data_Writer_Xml::$mime,

		);

		$format = isset($_GET['format']) && strlen($_GET['format']) < 16 ? $_GET['format'] : false;

		if(isset($formats[$format]))
		{
			$accept = $formats[$format];
		}
		else
		{
			$accept = PSX_Config::getRequestHeader('accept');
		}


		$writerType = null;

		switch(true)
		{
			case (stripos($accept, PSX_Data_Writer_Atom::$mime) !== false):

				$writerType = PSX_Data_WriterInterface::ATOM;

				break;

			case (stripos($accept, PSX_Data_Writer_Form::$mime) !== false):

				$writerType = PSX_Data_WriterInterface::FORM;

				break;

			default:
			case (stripos($accept, PSX_Data_Writer_Json::$mime) !== false):

				$writerType = PSX_Data_WriterInterface::JSON;

				break;

			case (stripos($accept, PSX_Data_Writer_Rss::$mime) !== false):

				$writerType = PSX_Data_WriterInterface::RSS;

				break;

			case (stripos($accept, PSX_Data_Writer_Xml::$mime) !== false):

				$writerType = PSX_Data_WriterInterface::XML;

				break;
		}

		return $writerType;
	}

	private function getWriter($writerType)
	{
		$writer = null;

		switch($writerType)
		{
			case PSX_Data_WriterInterface::ATOM:

				header('Content-type: ' . PSX_Data_Writer_Atom::$mime);

				$writer = new PSX_Data_Writer_Atom();

				break;

			case PSX_Data_WriterInterface::FORM:

				header('Content-type: ' . PSX_Data_Writer_Form::$mime);

				$writer = new PSX_Data_Writer_Form();

				break;

			case PSX_Data_WriterInterface::JSON:

				header('Content-type: ' . PSX_Data_Writer_Json::$mime);

				$writer = new PSX_Data_Writer_Json();

				break;

			case PSX_Data_WriterInterface::RSS:

				header('Content-type: ' . PSX_Data_Writer_Rss::$mime);

				$writer = new PSX_Data_Writer_Rss();

				break;

			case PSX_Data_WriterInterface::XML:

				header('Content-type: ' . PSX_Data_Writer_Xml::$mime);

				$writer = new PSX_Data_Writer_Xml();

				break;
		}

		return $writer;
	}
}

