<?php
/*
 *  $Id: Cache.php 272 2011-04-25 21:18:23Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Cache
 *
 * Provides a general caching mechanism. Here an example how you can use the
 * cache class in your module. As [key] you must provide a unique key if you
 * want cache a page you can use the variable $_SERVER['REQUEST_URI'] as $key
 * but you should control what values are passed to the page because else
 * someone can create uncontrollable many cache files by passing random
 * arguments so you should check whether the arguments are valid and if not you
 * set $cache->write = false so no cache is written.
 *
 * <code>
 * $cache = new PSX_Cache('[key]', new PSX_Cache_Handler_File(), $this->config);
 *
 * if(($content = $cache->load()) === false)
 * {
 * 	// here you should do some complex stuff to get the value for content
 * 	// so that it is worth to cache the content
 *
 * 	$content = 'test';
 *
 * 	$cache->write($content);
 * }
 *
 * echo $content;
 * </code>
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Cache
 * @version    $Revision: 272 $
 */
class PSX_Cache
{
	/**
	 * The cache key
	 *
	 * @var string
	 */
	public $key;

	/**
	 * The expire time of the cache in seconds
	 *
	 * @var integer
	 */
	public $expire;

	/**
	 * Whether to write the cache or not
	 *
	 * @var boolean
	 */
	public $write;

	/**
	 * The handler for the class
	 *
	 * @var psx_cache_ihandler
	 */
	private $handler;

	private $config;

	/**
	 * __construct
	 *
	 * To create an cache object we need an $key wich identifies the cache.
	 * The handler is an object of type psx_cache_ihandler wich is used to
	 * load and write the cache. Optional you can set the $expire time how
	 * long the cache remains if not set the default config cache expire
	 * time is used.
	 *
	 * @param string $key
	 * @param PSX_Cache_HandlerInterface $handler
	 * @param PSX_Config $config
	 * @param integer $expire
	 * @return void
	 */
	public function __construct($key, PSX_Cache_HandlerInterface $handler, PSX_Config $config, $expire = false)
	{
		$this->handler = $handler;
		$this->config  = $config;
		$this->key     = md5($key);
		$this->expire  = empty($expire) ? $this->config['psx_cache_expire'] : $expire;
		$this->write   = true;
	}

	/**
	 * load
	 *
	 * If caching is enabled we load the current cache from the
	 * handler if not we return false
	 *
	 * @return false|string
	 */
	public function load()
	{
		if($this->config['psx_cache_enabled'] === true)
		{
			return $this->handler->load($this->key, $this->expire);
		}
		else
		{
			return false;
		}
	}

	/**
	 * write
	 *
	 * Write the string $content to the cache by using the current
	 * handler.
	 *
	 * @param string $content
	 * @return void
	 */
	public function write($content)
	{
		if($this->write === true)
		{
			$this->handler->write($this->key, $content);
		}
	}
}