<?php
/*
 *  $Id: Dom.php 260 2011-03-30 20:00:01Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Dom
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Dom
 * @version    $Revision: 260 $
 */
class PSX_Dom extends DOMDocument
{
	/**
	 * Contains the DOMXpath object
	 *
	 * @var DOMXpath
	 */
	public $xp;

	/**
	 * An array of registered namspaces
	 *
	 * @var array
	 */
	public $ns;

	/**
	 * __construct
	 *
	 * Creates a new DOMDocument and DOMXpath object and add the root
	 * element if its not null.
	 *
	 * @param string $root
	 * @return void
	 */
	public function __construct($root = null)
	{
		parent::__construct('1.0', 'UTF-8');

		$this->xp = new DOMXpath($this);
		$this->ns = array();

		if($root !== null)
		{
			$this->add('/', self::node($root));
		}
	}

	/**
	 * parse
	 *
	 * Add to the xpath $path the nodes defined in the array format
	 *
	 * @param string $path
	 * @param array $content
	 * @return void
	 */
	public function parse($path, array $content)
	{
		$handle = $this->getNode($path);

		foreach($content as $node)
		{
			if(!isset($node['name']) || !is_array($node) || empty($node['name']))
			{
				throw new PSX_Dom_Exception('Xpath parse error you must set a name for the node!');
			}

			# values
			$hasChilds = false;

			if(isset($node['value']))
			{
				if(is_array($node['value']))
				{
					$element = $this->buildElement($node['name']);

					$hasChilds = true;
				}
				else
				{
					$element = $this->buildElement($node['name']);

					$cdata = $this->createCDATASection($node['value']);

					$element->appendChild($cdata);
				}
			}
			else
			{
				$element = $this->buildElement($node['name']);
			}

			$handle->appendChild($element);


			$currentPath = $path . '/' . $node['name'];

			$currentNode = $this->xpathQuery($currentPath);

			# attributes
			if(isset($node['attr']) && is_array($node['attr']))
			{
				foreach($node['attr'] as $key => $value)
				{
					if(!$currentNode->hasAttribute($key))
					{
						$currentNode->setAttribute($key, $value);
					}
				}
			}

			if($hasChilds === true)
			{
				$this->parse($currentPath, $node['value']);
			}
		}
	}

	/**
	 * buildElement
	 *
	 * Creates an new DOMElement. If an error occured it returns false else
	 * an DOMElement
	 *
	 * @param string $name
	 * @param false|string $value
	 * @return false|DOMElement
	 */
	private function buildElement($name, $value = false)
	{
		try
		{
			# namespace handling
			$ns = false;

			if(substr_count($name, ':') == 1)
			{
				$n = explode(':', $name);

				$prefix = $n[0];

				if(!array_key_exists($prefix, $this->ns))
				{
					throw new psx_dom_exception('Namespace ' . $prefix . ' not registered');
				}
				else
				{
					$uri = $this->ns[$prefix];

					$ns = true;
				}
			}

			# create element
			if($ns === true)
			{
				if($value === false)
				{
					$element = $this->createElementNS($uri, $name);
				}
				else
				{
					$element = $this->createElementNS($uri, $name, $value);
				}
			}
			else
			{
				if($value === false)
				{
					$element = $this->createElement($name);
				}
				else
				{
					$element = $this->createElement($name, $value);
				}
			}

			return $element;
		}
		catch(DOMException $e)
		{
			throw new psx_dom_exception('Error while create an element: ' . $e->getMessage());
		}
	}

	/**
	 * add
	 *
	 * Wrapper function to the parse method
	 *
	 * @param string $path
	 * @param array $content
	 * @return void
	 */
	public function add($path, array $content)
	{
		$this->parse($path, $content);
	}

	/*
	public function replace($path, $content)
	{
		if(!($handle = $this->xpath_query($path)))
		{
			throw new psx_exception('DOM: invalid xpath query!');
		}

		$this->remove($path);

		$this->add($path, $content);
	}

	public function remove($path)
	{
		if(!($handle = $this->xpath_query($path)))
		{
			throw new psx_exception('DOM: invalid xpath query!');
		}

		$this->removeChild($handle);
	}
	*/

	/**
	 * toXml
	 *
	 * Returns the node from $path as an XML string. If path is not set the
	 * complete document will be converted to an XML string
	 *
	 * @param string $path
	 * @return string
	 */
	public function toXml($path = '/')
	{
		$node = $this->getNode($path);

		$xml  = $this->saveXml($node);

		return $xml;
	}

	/**
	 * getNode
	 *
	 * Return the node from $path as DOMNode.
	 *
	 * @param string $path
	 * @return DOMNode
	 */
	public function getNode($path)
	{
		if(!($node = $this->xpathQuery($path)))
		{
			throw new PSX_Dom_Exception('Invalid xpath query!');
		}
		else
		{
			return $node;
		}
	}

	/**
	 * setAttribute
	 *
	 * Adds the attributes defined in $attr to the node $path
	 *
	 * @param string $path
	 * @param array $attr
	 * @return void
	 */
	public function setAttribute($path, array $attr)
	{
		if(!($handle = $this->xpathQuery($path)))
		{
			throw new PSX_Dom_Exception('Invalid xpath query!');
		}

		foreach($attr as $key => $value)
		{
			if(!$handle->hasAttribute($key))
			{
				$handle->setAttribute($key, $value);
			}
		}
	}

	/**
	 * xpathQuery
	 *
	 * Executes an xpath to the current DOMDocument. If the result has more
	 * then one node it will return always the last
	 *
	 * @param string $path
	 * @return DOMNode
	 */
	public function xpathQuery($path)
	{
		$result = $this->xp->query($path);

		$last_node = null;

		if($result->length > 0)
		{
			foreach($result as $node)
			{
				$last_node = $node;
			}
		}
		else
		{
			return false;
		}

		if($last_node === null)
		{
			return false;
		}
		else
		{
			return $last_node;
		}
	}

	/**
	 * registerNs
	 *
	 * Registers a new namspace to the document. Returns true on success and
	 * false on failure
	 *
	 * @param string $prefix
	 * @param string $uril
	 * @return boolean
	 */
	public function registerNs($prefix, $uri)
	{
		if(!array_key_exists($prefix, $this->ns) && $this->xp->registerNamespace($prefix, $uri))
		{
			$this->ns[$prefix] = $uri;

			# create dummy attribute to insert xmlns:$prefix namespace
			# then delete the dummy node. if you found a better way to
			# insert a namespace at the root node contact me
			$psx = $this->xpathQuery('/psx');

			$key = $prefix . ':temp';

			$psx->setAttributeNS($uri, $key, false);

			$psx->removeAttribute($key);

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * insertXml
	 *
	 * Insert raw xml to the node $path and return true on success
	 *
	 * @param string $path
	 * @param string $xml
	 * @return boolean
	 */
	public function insertXml($path, $xml)
	{
		if(!($node = $this->xpathQuery($path)))
		{
			throw new psx_dom_exception('Invalid xpath query');
		}
		else
		{
			$domFragment = $this->createDocumentFragment();

			$domFragment->appendXML($xml);

			$node->appendChild($domFragment);

			return true;
		}
	}

	/**
	 * insertDom
	 *
	 * Insert a DOMNode to the node $path and return true on success
	 *
	 * @param string $path
	 * @param DOMNode $dom
	 * @return boolean
	 */
	public function insertDom($path, DOMNode $dom)
	{
		if(!($node = $this->xpathQuery($path)))
		{
			throw new PSX_Dom_Exception('Invalid xpath query');
		}
		else
		{
			$importNode = $this->importNode($dom, true);

			$node->appendChild($importNode);

			return true;
		}
	}

	/**
	 * lists
	 *
	 * Returns an array in a format so that you can parse it with the parse
	 * method
	 *
	 * @param string $block
	 * @param string $content
	 * @return array
	 */
	public static function lists($block, $content)
	{
		$list = array();
		$node = array();

		foreach($content as $name => $value)
		{
			array_push($node, array(

				'name'  => $name,
				'value' => $value

			));
		}

		$list = self::node($block, $node);

		return $list;
	}

	/**
	 * node
	 *
	 * Returns a node in an array format so that you can parse it with the
	 * parse method
	 *
	 * @param string $name
	 * @param string $value
	 * @param string $attr
	 * @return array
	 */
	public static function node($name, $value = false, $attr = false)
	{
		$n = array();
		$p = array();

		$p['name'] = $name;

		if($value !== false) { $p['value'] = $value; }
		if($attr  !== false) { $p['attr']  = $attr;  }

		$n[] = $p;

		return $n;
	}
}
