<?php
/*
 *  $Id: Loader.php 275 2011-05-01 11:26:41Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Loader
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Loader
 * @version    $Revision: 275 $
 */
class PSX_Loader
{
	private $config;
	private $template;

	private $loaded;
	private $routes;

	public function __construct(PSX_Config $config, PSX_Template $template)
	{
		$this->config   = $config;
		$this->template = $template;

		$this->loaded   = array();
		$this->routes   = array();
	}

	public function load($path)
	{
		if(($rewritePath = $this->getRoute($path)) !== false)
		{
			$path = $rewritePath;
		}

		if(!in_array($path, $this->loaded))
		{
			list($path, $file, $class, $method, $is_static, $uriFragments) = $this->parsePath($path);

			if($is_static === false)
			{
				$handle = new $class($this->config, $this->template, $path, $uriFragments);

				if($handle instanceof PSX_ModuleAbstract)
				{
					$this->loaded[] = $path;

					$handle->_ini();

					if($handle instanceof PSX_Module_PrivateInterface)
					{
						// we dont call any method if the class is private
					}
					else
					{
						if(!empty($method))
						{
							call_user_func(array($handle, $method));
						}
					}

					return $handle;
				}
				else
				{
					throw new PSX_Loader_Exception('Class it is not an instance of PSX_ModuleAbstract');
				}
			}
			else
			{
				if(!empty($method))
				{
					$this->loaded[] = $path;

					call_user_func($class . '::' . $method);

					return null;
				}
			}
		}

		return false;
	}

	/**
	 * parse url
	 * index.php?x=[path/to/the/file]/class/[method]
	 *
	 * the brackets in [] are optional. the [method] is only called when it
	 * is aviable. All values up to the questionmark are saved as uri
	 * fragment. You can access these values in your module with
	 * $this->getUriFragment() wich returns the values as an array where
	 * the values are exploded after '/'
	 */
	public function parsePath($x, $deep = 0)
	{
		$x   = empty($x) ? $this->config['psx_module_default'] : $x;
		$x   = trim($x, '/');

		$pos = strpos($x, '::');
		$paa = $pos !== false ? substr($x, $pos + 2) : false;
		$x   = $pos !== false ? substr($x, 0, $pos)  : $x;
		$fpc = self::getFPC($x);

		if($fpc !== false)
		{
			list($file, $path, $class) = $fpc;

			require_once($file);

			if(!class_exists($class, false))
			{
				throw new PSX_Loader_Exception('Class ' . $class . ' doesnt exist in ' . $file);
			}


			// control whether the method exists or not
			$method       = false;
			$isStatic     = false;
			$rest         = substr($x, strlen($path) + 1);
			$classMethods = get_class_methods($class);
			$reserved     = array('onLoad', 'onPost', 'onPut', 'onDelete', 'onGet');

			if(!empty($paa))
			{
				$isStatic = true;

				if(empty($rest))
				{
					$method = self::getPart($paa);

					if(in_array($method, $classMethods) && !in_array($method, $reserved))
					{
						$rest = substr($paa, strlen($method) + 1);
					}
					else
					{
						throw new PSX_Loader_Exception('Call to an undefined static method');
					}
				}
			}
			else
			{
				$isStatic = false;

				if(!empty($rest))
				{
					$method = self::getPart($rest);

					if(in_array($method, $classMethods) && !in_array($method, $reserved))
					{
						$rest = substr($rest, strlen($method) + 1);
					}
					else
					{
						$method = false;
					}
				}
			}


			// if we have no method look for an index
			if($isStatic === false && $method === false)
			{
				if(in_array('index', $classMethods))
				{
					$method = 'index';
				}
			}


			// get uri fragments
			$uriFragments = array();

			if(!empty($rest))
			{
				$uriFragments = explode('/', trim($rest, '/'));
			}
		}
		else
		{
			if($deep == 0)
			{
				$x = $this->config['psx_module_default'] . '/' . $x;

				return $this->parsePath($x, ++$deep);
			}
			else
			{
				//header('HTTP/1.1 404 Not Found');

				throw new PSX_Loader_Exception('Unkown module!');
			}
		}

		return array(

			$path,
			$file,
			$class,
			$method,
			$isStatic,
			$uriFragments,

		);
	}

	public static function getPart($path)
	{
		$pos = strpos($path, '/');

		if($pos === false)
		{
			return $path;
		}
		else
		{
			return substr($path, 0, $pos);
		}
	}

	public static function getFPC($path)
	{
		$path = trim($path, '/');

		if(is_file(PSX_PATH_MODULE . '/' . $path . '.php'))
		{
			$file = PSX_PATH_MODULE . '/' . $path . '.php';
			$pos  = strrpos($path, '/');

			if($pos === false)
			{
				$class = $path;
			}
			else
			{
				$class = substr($path, $pos + 1);
			}

			return array($file, $path, $class);
		}
		elseif(is_file(PSX_PATH_MODULE . '/' . $path . '/' . 'index.php'))
		{
			$file  = PSX_PATH_MODULE . '/' . $path . '/' . 'index.php';
			$class = 'index';

			return array($file, $path, $class);
		}
		else
		{
			$pos  = strrpos($path, '/');
			$file = substr($path, 0, $pos);

			if($pos === false)
			{
				return false;
			}
			else
			{
				return self::getFPC($file);
			}
		}
	}

	public function addRoute($path, $module)
	{
		$key = md5($path);

		$this->routes[$key] = $module;
	}

	public function getRoute($path)
	{
		$key = md5($path);

		return isset($this->routes[$key]) ? $this->routes[$key] : false;
	}
}


