<?php
/*
 *  $Id: OpenId.php 336 2011-10-10 16:29:03Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_OpenId
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_OpenId
 * @version    $Revision: 336 $
 */
class PSX_OpenId
{
	public static $xriGlobalContextSymbol = array('=', '@', '+', '$', '!', '(');

	private $types  = array();
	private $data   = array();
	private $params = array();

	private $claimedId;
	private $identity;
	private $returnTo;

	private $lastError = false;

	private $http;
	private $trustRoot;

	public function __construct(PSX_Http $http, $trustRoot)
	{
		$this->http      = $http;
		$this->trustRoot = $trustRoot;
	}

	public function initialize($identity, $returnTo)
	{
		$this->claimedId = $identity;
		$this->identity  = $this->discovery($identity);
		$this->returnTo  = $returnTo;
	}

	/**
	 * add
	 *
	 * piggybacks extension parameters for an openid request. if we have
	 * discovered an XRDS document we check whether the service support the
	 * extension if we have no informations what types are supported by the
	 * endpoint we add the params and hope that the server support it
	 *
	 * @param psx_net_openid_iextension $extension
	 * @return void
	 */
	public function add(PSX_OpenId_ExtensionInterface $extension)
	{
		if(empty($this->types))
		{
			$this->params+= $extension->getParams();
		}
		else
		{
			if(in_array($extension->getNs(), $this->types))
			{
				$this->params+= $extension->getParams();
			}
			else
			{
				throw new PSX_OpenId_Exception('The extension ' . $extension->getNs() . ' is not supported by the service');
			}
		}
	}

	public function redirect(array $overrideParams = array())
	{
		$params = array(

			'openid.ns'         => PSX_OpenId_ProviderAbstract::NS,
			'openid.mode'       => 'checkid_setup',

			'openid.return_to'  => $this->returnTo,
			'openid.trust_root' => $this->trustRoot,

		) + $this->params;


		$identity = (string) $this->identity->getLocalId();

		if(!empty($identity))
		{
			$params['openid.claimed_id'] = $this->claimedId;
			$params['openid.identity']   = $identity;
		}
		else
		{
			$params['openid.claimed_id'] = $this->claimedId;
			$params['openid.identity']   = $this->claimedId;
		}


		if(!empty($overrideParams))
		{
			foreach($overrideParams as $k => $v)
			{
				$params[$k] = $v;
			}
		}


		$url = $this->identity->getServer();
		$url->addParams($params);

		header('HTTP/1.1 302 Found');
		header('Location: ' . $url);

		exit;
	}

	public function verify()
	{
		$mode = isset($_GET['openid_mode']) ? $_GET['openid_mode'] : false;

		if($mode == 'cancel')
		{
			throw new PSX_OpenId_Exception('User has canceled the request');
		}

		if($mode == 'error')
		{
			$msg = isset($_GET['openid_error']) ? $_GET['openid_error'] : false;

			throw new PSX_OpenId_Exception(!empty($msg) ? $msg : 'An error occured but no message was specified');
		}

		if($mode == 'id_res')
		{
			$openidIdentity    = isset($_GET['openid_identity'])     ? $_GET['openid_identity']     : false;
			$openidOpEndpoint  = isset($_GET['openid_op_endpoint'])  ? $_GET['openid_op_endpoint']  : false;
			$openidAssocHandle = isset($_GET['openid_assoc_handle']) ? $_GET['openid_assoc_handle'] : false;
			$openidSigned      = isset($_GET['openid_signed'])       ? $_GET['openid_signed']       : false;
			$openidSig         = isset($_GET['openid_sig'])          ? $_GET['openid_sig']          : false;


			if(!empty($openidOpEndpoint))
			{
				$openidOpEndpoint = new PSX_Url($openidOpEndpoint);
			}
			else
			{
				throw new PSX_OpenID_Exception('Endpoint not set');
			}


			// verify the identity
			$this->params = array(

				'openid.assoc_handle' => $openidAssocHandle,
				'openid.signed'       => $openidSigned,
				'openid.sig'          => $openidSig,
				'openid.mode'         => 'check_authentication',

			);


			# you should note that php replaces . (dot) in the query
			# fragment with _ (underscore) i.e. index.php?openid.foo=bar
			# can you access with $_GET['openid_foo']

			if(!empty($openidSigned))
			{
				$signed = $openidSigned;
				$parts  = explode(',', $signed);

				foreach($parts as $part)
				{
					$k = 'openid_' . str_replace('.', '_', $part);
					$v = isset($_GET[$k]) ? $_GET[$k] : false;

					if($v !== false && $k != 'openid_mode')
					{
						$this->params['openid.' . $part] = $v;
					}

					$this->data[$k] = urldecode($v);
				}
			}


			$request   = new PSX_Http_PostRequest($openidOpEndpoint, array(), $this->params);
			$response  = $this->http->request($request);
			$lastError = $this->http->getLastError();

			if(empty($lastError))
			{
				if($response->getCode() == 200)
				{
					$data = PSX_OpenId::keyValueDecode($response->getBody());

					if(isset($data['error']))
					{
						throw new PSX_OpenID_Exception($data['error']);
					}

					if(isset($data['is_valid']) && $data['is_valid'] == 'true')
					{
						// set identity
						$claimedId = isset($this->data['openid_claimed_id'])  ? $this->data['openid_claimed_id']  : null;
						$server    = isset($this->data['openid_op_endpoint']) ? $this->data['openid_op_endpoint'] : null;
						$localId   = isset($this->data['openid_identity'])    ? $this->data['openid_identity']    : null;

						if(!empty($claimedId) && !empty($server) && !empty($localId))
						{
							$this->claimedId = $claimedId;
							$this->identity  = new PSX_OpenId_Identity($server, $localId);

							return true;
						}
						else
						{
							throw new PSX_OpenID_Exception('Missing data in response');
						}
					}
					else
					{
						throw new PSX_OpenID_Exception('Identity is not valid');
					}
				}
				else
				{
					throw new PSX_OpenID_Exception('Invalid response code received "' . $response->getCode() . ' ' . $response->getMessage() . '"');
				}
			}
			else
			{
				throw new PSX_OpenID_Exception($lastError);
			}
		}
	}

	public function hasExtension($ns)
	{
		return in_array($ns, $this->types);
	}

	/**
	 * getData
	 *
	 * Returns the data that we have received via the callback
	 *
	 * @return array
	 */
	public function getData()
	{
		return $this->data;
	}

	/**
	 * getTypes
	 *
	 * Returns an array with all service types that the discovered service
	 * support (namespaces)
	 *
	 * @return array
	 */
	public function getTypes()
	{
		return $this->types;
	}

	public function getClaimedId()
	{
		return $this->claimedId;
	}

	public function getIdentity()
	{
		return $this->identity;
	}

	public function getReturnTo()
	{
		return $this->returnTo;
	}

	/**
	 * getIdentifier
	 *
	 * Returns an identifier of the user wich can be used to identify this
	 * particular user.
	 *
	 * @return string
	 */
	public function getIdentifier()
	{
		if($this->identity instanceof PSX_OpenId_Identity)
		{
			$identifier = $this->identity->getLocalId() === null ? $this->getClaimedId() : $this->identity->getLocalId();

			return self::normalizeIdentifier($identifier);
		}
		else
		{
			throw new PSX_OpenId_Exception('Identity not discovered');
		}
	}

	/**
	 * discovery
	 *
	 * Discovers an openid identity (URI or XRI). It uses the xri.net web
	 * service to resolve XRIs. Returns either an PSX_OpenId_Identity
	 *
	 * @param string $identity
	 * @return PSX_OpenId_Identity
	 */
	private function discovery($identity)
	{
		$identity = $this->normalizeUri($identity);

		// we could get the identity via XRI
		if($identity instanceof PSX_OpenId_Identity)
		{
			return $identity;
		}
		// its an url wich we must discover
		else if($identity instanceof PSX_Url)
		{
			# YADIS discovery
			$discoveredIdentity = $this->fetchXrds($identity);

			# HTML based discovery
			if($discoveredIdentity === false)
			{
				$request  = new PSX_Http_GetRequest($identity);
				$request->setFollowLocation(true);

				$response = $this->http->request($request);

				return $this->htmlBasedDiscovery($response->getBody());
			}
			else
			{
				return $discoveredIdentity;
			}
		}
		else
		{
			throw new PSX_OpenId_Exception('Invalid openid identity');
		}
	}

	/**
	 * fetchXrds
	 *
	 * We make a YADIS dicovery on the url. If we have found an fitting
	 * service the method returns the endpoint uri and if available a
	 * claimed id else it returns false
	 *
	 * @param string $url
	 * @return array
	 */
	private function fetchXrds(PSX_Url $url)
	{
		$yadis = new PSX_Yadis($this->http);
		$xrds  = $yadis->discover($url);

		if($xrds !== false && isset($xrds->service))
		{
			// OP Identifier Element
			foreach($xrds->service as $service)
			{
				if(in_array('http://specs.openid.net/auth/2.0/server', $service->getType()))
				{
					$this->types = $service->getType();

					$identity = new PSX_OpenId_Identity($service->getUri());

					return $identity;
				}
			}

			// Claimed Identifier Element
			foreach($xrds->service as $service)
			{
				if(in_array('http://specs.openid.net/auth/2.0/signon', $service->getType()))
				{
					$this->types = $service->getType();

					$identity = new PSX_OpenId_Identity($service->getUri(), $service->getLocalId());

					return $identity;
				}
			}
		}

		return false;
	}

	/**
	 * normalizeUri
	 *
	 * Resolves the $uri either to an PSX_OpenId_Identity if its an XRI or
	 * to an PSX_Url
	 *
	 * @return PSX_OpenId_Identity|PSX_Url
	 */
	public function normalizeUri($uri)
	{
		if(!empty($uri))
		{
			$uri   = strtolower($uri);
			$isXri = false;

			if(substr($uri, 0, 6) == 'xri://')
			{
				$uri   = substr($uri, 6);
				$isXri = true;
			}

			if(in_array($uri[0], self::$xriGlobalContextSymbol))
			{
				$isXri = true;
			}

			if($isXri !== true)
			{
				if(substr($uri, 0, 7) != 'http://' && substr($uri, 0, 8) != 'https://')
				{
					$uri = 'http://' . $uri;
				}

				$url = new PSX_Url($uri);

				if($url->getScheme() == 'http' || $url->getScheme() == 'https')
				{
					return $url;
				}
				else
				{
					throw new PSX_OpenId_Exception('Unknown protocol in identity');
				}
			}
			else
			{
				return $this->discoverXriIdentity($uri);
			}
		}

		return false;
	}

	public function discoverXriIdentity($identity)
	{
		// we use the XRI resolver at xri.net
		$request  = new PSX_Http_GetRequest(new PSX_Url('http://xri.net/' . $identity));
		$response = $this->http->request($request);

		// we accept all 3xx redirect status codes if we dont
		// get redirected we couldnt resolve the XRI
		$header = $response->getHeader();

		if($response->getCode() >= 300 && $response->getCode() < 400 && isset($header['location']))
		{
			// we make an YADIS request to the resolved URI to get
			// an XRDS document
			$location = new PSX_Url($header['location']);

			return $this->fetchXrds($location);
		}
		else
		{
			throw new psx_net_openid_exception('Couldnt resolve XRI');
		}
	}

	private function htmlBasedDiscovery($html)
	{
		$parse = new PSX_Html_Parse($html);


		$element = new PSX_Html_Parse_Element('link', array('rel' => 'openid2.provider'));
		$server  = $parse->fetchAttrFromHead($element, 'href');

		if(empty($server))
		{
			$element->setAttributes(array('rel' => 'openid.server'));

			$server = $parse->fetchAttrFromHead($element, 'href');
		}


		$element = new PSX_Html_Parse_Element('link', array('rel' => 'openid2.local_id'));
		$localId = $parse->fetchAttrFromHead($element, 'href');

		if(empty($localId))
		{
			$element->setAttributes(array('rel' => 'openid.delegate'));

			$localId = $parse->fetchAttrFromHead($element, 'href');
		}


		if(!empty($server))
		{
			$identity = new PSX_OpenId_Identity($server, $localId);

			return $identity;
		}
		else
		{
			throw new PSX_OpenId_Exception('Couldnt find server in identifier');
		}
	}

	public static function keyValueEncode(array $data)
	{
		$content = '';

		foreach($data as $k => $v)
		{
			$content.= $k . ':' . $v . "\n";
		}

		return $content;
	}

	public static function keyValueDecode($data)
	{
		$params = array();
		$lines  = explode("\n", $data);

		foreach($lines as $line)
		{
			$line = trim($line);

			if(!empty($line))
			{
				$pair = explode(':', $line, 2);

				if(isset($pair[0]) && isset($pair[1]))
				{
					$k = str_replace(array("\n", ':'), '', trim($pair[0]));
					$v = str_replace(array("\n"),      '', trim($pair[1]));

					$params[$k] = $v;
				}
			}
		}

		return $params;
	}

	public static function normalizeIdentifier($identifier)
	{
		$normalize  = '';
		$identifier = trim(trim(strtolower($identifier)), '/');

		$url = parse_url($identifier);

		if(isset($url['user']))
		{
			$normalize.= $url['user'] . '@';
		}

		if(isset($url['host']))
		{
			$normalize.= $url['host'];
		}
		else
		{
			return false;
		}

		if(isset($url['path']))
		{
			$normalize.= $url['path'];
		}

		return $normalize;
	}
}
