<?php
/*
 *  $Id: ResRequest.php 346 2011-10-22 23:10:39Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_OpenId_Provider_Data_ResRequest
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_OpenId
 * @version    $Revision: 346 $
 */
class PSX_OpenId_Provider_Data_ResRequest extends PSX_Data_RecordAbstract
{
	public $opEndpoint;
	public $claimedId;
	public $identity;
	public $returnTo;
	public $responseNonce;
	public $invalidateHandle;
	public $assocHandle;
	public $signed;
	public $sig;

	private $params;
	private $map = array(

		'opEndpoint'       => 'op_endpoint',
		'claimedId'        => 'claimed_id',
		'identity'         => 'identity',
		'returnTo'         => 'return_to',
		'responseNonce'    => 'response_nonce',
		'invalidateHandle' => 'invalidate_handle',
		'assocHandle'      => 'assoc_handle',
		'signed'           => 'signed',
		'sig'              => 'sig',

	);

	public function getName()
	{
		return 'request';
	}

	public function getFields()
	{
		$fields = array();

		foreach($this->map as $k => $v)
		{
			$value = $this->$k;

			if(!empty($value))
			{
				$fields[$v] = $value;
			}
		}

		return $fields;
	}

	public function setOpEndpoint($opEndpoint)
	{
		$this->opEndpoint = new PSX_Url($opEndpoint);
	}

	public function setClaimedId($claimedId)
	{
		$this->claimedId = $claimedId;
	}

	public function setIdentity($identity)
	{
		$this->identity = $identity;
	}

	public function setReturnTo($returnTo)
	{
		$this->returnTo = new PSX_Url($returnTo);
	}

	public function setResponseNonce($responseNonce)
	{
		$this->responseNonce = $responseNonce;
	}

	public function setInvalidateHandle($invalidateHandle)
	{
		$this->invalidateHandle = $invalidateHandle;
	}

	public function setAssocHandle($assocHandle)
	{
		$this->assocHandle = $assocHandle;
	}

	public function setSigned($signed)
	{
		$mustSigned = array('op_endpoint', 'return_to', 'response_nonce', 'assoc_handle', 'claimed_id', 'identity');
		$signed     = explode(',', $signed);

		if(count(array_diff(array_intersect($mustSigned, $signed), $mustSigned)) > 0)
		{
			throw new PSX_Data_Exception('You must sign at least: ' . implode(',', $mustSigned));
		}

		$this->signed = $signed;
	}

	public function setSig($sig)
	{
		$this->sig = $sig;
	}

	public function getOpEndpoint()
	{
		return $this->opEndpoint;
	}

	public function getClaimedId()
	{
		return $this->claimedId;
	}

	public function getIdentity()
	{
		return $this->identity;
	}

	public function getReturnTo()
	{
		return $this->returnTo;
	}

	public function getResponseNonce()
	{
		return $this->responseNonce;
	}

	public function getInvalidateHandle()
	{
		return $this->invalidateHandle;
	}

	public function getAssocHandle()
	{
		return $this->assocHandle;
	}

	public function getSigned()
	{
		return $this->signed;
	}

	public function getSig()
	{
		return $this->sig;
	}

	public function buildParams()
	{
		// build basic params
		$params = array(

			'ns'             => PSX_OpenId_ProviderAbstract::NS,
			'mode'           => 'id_res',
			'op_endpoint'    => strval($this->getOpEndpoint()),
			'return_to'      => strval($this->getReturnTo()),
			'response_nonce' => $this->getResponseNonce(),
			'assoc_handle'   => $this->getAssocHandle(),

		);

		$claimedId = $this->getClaimedId();
		$identity  = $this->getIdentity();

		if(!empty($claimedId) && !empty($identity))
		{
			$params['claimed_id'] = $claimedId;
			$params['identity']   = $identity;
		}


		// merge extensions
		$params = array_merge($params, $this->getParams());


		return $params;
	}

	public function buildSignature($secret, $hashAlgo = 'sha256')
	{
		$params = $this->buildParams();
		$data   = array();

		$this->signed = array_keys($params);

		if(empty($this->signed))
		{
			throw new PSX_Data_Exception('No values are signed');
		}

		foreach($this->signed as $k)
		{
			$v = isset($params[$k]) ? $params[$k] : null;

			if($v !== null && $k != 'mode')
			{
				$data[$k] = $v;
			}
		}

		$baseString = PSX_OpenId::keyValueEncode($data);
		$signature  = base64_encode(hash_hmac($hashAlgo, $baseString, $secret, true));

		$this->sig = $signature;

		return $signature;
	}

	public function redirect()
	{
		if(empty($this->sig))
		{
			throw new PSX_Data_Exception('No signature was set');
		}


		// build signature
		$params           = $this->buildParams();
		$params['signed'] = implode(',', $this->getSigned());
		$params['sig']    = $this->getSig();


		// add params to url
		$url = $this->getReturnTo();

		if(empty($url))
		{
			throw new PSX_Data_Exception('No return_to url was set');
		}

		foreach($params as $k => $v)
		{
			$url->addParam('openid.' . $k, $v);
		}


		// redirect
		header('Location: ' . strval($url));
		exit;
	}

	public function setParams(array $params)
	{
		$this->params = $params;
	}

	public function getParams()
	{
		return $this->params;
	}

	public function import(PSX_Data_ReaderResult $result)
	{
		switch($result->getType())
		{
			case PSX_Data_ReaderInterface::RAW:

				$params = array();

				parse_str($result->getData()->getBody(), $params);

				if(isset($params['openid_op_endpoint']))
				{
					$this->setOpEndpoint($params['openid_op_endpoint']);

					unset($params['openid_op_endpoint']);
				}
				else
				{
					throw new PSX_Data_Exception('OP endpoint not set');
				}

				if(isset($params['openid_claimed_id']))
				{
					$this->setClaimedId($params['openid_claimed_id']);

					unset($params['openid_claimed_id']);
				}

				if(isset($params['openid_identity']))
				{
					$this->setIdentity($params['openid_identity']);

					unset($params['openid_identity']);
				}

				if(isset($params['openid_return_to']))
				{
					$this->setReturnTo($params['openid_return_to']);

					unset($params['openid_return_to']);
				}
				else
				{
					throw new PSX_Data_Exception('Return to not set');
				}

				if(isset($params['openid_response_nonce']))
				{
					$this->setResponseNonce($params['openid_response_nonce']);

					unset($params['openid_response_nonce']);
				}
				else
				{
					throw new PSX_Data_Exception('Response nonce not set');
				}

				if(isset($params['openid_invalidate_handle']))
				{
					$this->setInvalidateHandle($params['openid_invalidate_handle']);

					unset($params['openid_invalidate_handle']);
				}

				if(isset($params['openid_assoc_handle']))
				{
					$this->setAssocHandle($params['openid_assoc_handle']);

					unset($params['openid_assoc_handle']);
				}
				else
				{
					throw new PSX_Data_Exception('Assoc handle not set');
				}

				if(isset($params['openid_signed']))
				{
					$this->setSigned($params['openid_signed']);

					unset($params['openid_signed']);
				}
				else
				{
					throw new PSX_Data_Exception('Signed not set');
				}

				if(isset($params['openid_sig']))
				{
					$this->setSig($params['openid_sig']);

					unset($params['openid_sig']);
				}
				else
				{
					throw new PSX_Data_Exception('Sig not set');
				}


				// import signed parameters
				$signed = $this->getSigned();
				$data   = array();

				foreach($signed as $s)
				{
					$k = 'openid_' . str_replace('.', '_', $s);
					$v = isset($params[$k]) ? $params[$k] : null;

					if($v !== null && $k != 'openid_mode')
					{
						$data[$s] = $v;
					}
				}

				$this->setParams($data);

				break;

			default:

				throw new PSX_Data_Exception('Can only import data from reader Raw');

				break;
		}
	}

	public function export(PSX_Data_WriterResult $result)
	{
		switch($result->getType())
		{
			case PSX_Data_WriterInterface::FORM:
			case PSX_Data_WriterInterface::JSON:
			case PSX_Data_WriterInterface::XML:

				return $this->getFields();

				break;
		}
	}
}