<?php
/*
 *  $Id: ProviderAbstract.php 333 2011-10-10 14:06:34Z k42b3.x@googlemail.com $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_OpenId_ProviderAbstract
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_OpenId
 * @version    $Revision: 333 $
 */
abstract class PSX_OpenId_ProviderAbstract extends PSX_ApiAbstract
{
	const NS = 'http://specs.openid.net/auth/2.0';

	protected function handle()
	{
		$ns   = isset($_REQUEST['openid_ns'])   ? $_REQUEST['openid_ns'] : null;
		$mode = isset($_REQUEST['openid_mode']) ? $_REQUEST['openid_mode'] : null;

		if($ns != self::NS)
		{
			throw new PSX_OpenId_Provider_Exception('Namespace not set or invalid');
		}

		switch($mode)
		{
			case 'associate':

				throw new PSX_OpenId_Provider_Exception('Not implemented yet');

				break;

			case 'checkid_immediate':
			case 'checkid_setup':

				if($this->isAuthenticated())
				{
					$request = new PSX_OpenId_Provider_Data_SetupRequest();
					$request->import($this->getRequest(PSX_Data_ReaderInterface::RAW));

					$this->onCheckidSetup($request);
				}
				else
				{
					$this->doLogin();
				}

				break;

			case 'check_authentication':

				try
				{
					$request = new PSX_OpenId_Provider_Data_ResRequest();
					$request->import($this->getRequest(PSX_Data_ReaderInterface::RAW));

					$auth = $this->onCheckAuthentication($request->getAssocHandle());

					if($auth instanceof PSX_OpenId_Provider_Data_Authentication)
					{
						echo PSX_OpenId::keyValueEncode(array(

							'ns'       => self::NS,
							'is_valid' => strcmp($auth->getSig(), $request->buildSignature($auth->getSecret())) === 0 ? 'true' : 'false',

						));
					}
					else
					{
						throw new PSX_OpenId_Provider_Exception('Invalid association');
					}
				}
				catch(Exception $e)
				{
					echo PSX_OpenId::keyValueEncode(array(

						'ns'    => self::NS,
						'error' => $e->getMessage() . "\n" . $e->getTraceAsString(),

					));
				}

				exit;

				break;

			default:

				throw new PSX_OpenId_Provider_Exception('Invalid mode');

				break;
		}
	}

	/**
	 * doLogin
	 *
	 * Redirects or shows the current user an login form where he can
	 * authenticate. If the authentication was successful the user should be
	 * redirected to this page to proceed the checkid_setup request
	 *
	 * @return void
	 */
	abstract public function doLogin();

	/**
	 * isAuthenticated
	 *
	 * Returns true or false whether the user is authenticated or not
	 *
	 * @return boolean
	 */
	abstract public function isAuthenticated();

	/**
	 * onCheckidSetup
	 *
	 * Is called if a user peforms an openid checkid_setup request
	 *
	 * @return void
	 */
	abstract public function onCheckidSetup(PSX_OpenId_Provider_Data_SetupRequest $request);

	/**
	 * getAuthentication
	 *
	 * Returns the authentication response wich was previously send to the user
	 * by the assoc_handle.
	 *
	 * @return PSX_OpenId_Data_Authentication
	 */
	abstract public function onCheckAuthentication($assocHandle);

	/**
	 * getExtension
	 *
	 * Parses the $params array for extensions with the namespace $ns. The
	 * $params should be the values of $_GET or $_POST where the dot is replaced
	 * with an underscore
	 *
	 * @return array
	 */
	public static function getExtension(array $params, $ns)
	{
		$values = array();
		$alias  = null;

		// find alias
		foreach($params as $k => $v)
		{
			if($v == $ns)
			{
				$alias = substr($k, 10);
			}
		}

		if(!empty($alias))
		{
			// get values
			$len = strlen($alias);

			foreach($params as $k => $v)
			{
				if(substr($k, 0, 7 + $len) == 'openid_' . $alias)
				{
					$key = substr($k, 8 + $len);

					if(!empty($key))
					{
						$values[$key] = $v;
					}
				}
			}
		}

		return $values;
	}
}

