<?php
/*
 *  $Id: Registry.php 260 2011-03-30 20:00:01Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Registry
 *
 * This class is for managing all objects in your modules. The registry is
 * available in each module. You can register an object via the method
 * set(Object) and receive an object with the method get(path). The classes are
 * stored with their class names i.e:
 * <code>
 * // we save an instance of the psx loader
 * $registry->set(new PSX_Loader());
 *
 * $registry[] = new PSX_Loader(); // alternative
 *
 *
 * // we can get the object either via the full qualified name
 * $registry->get('PSX/Loader')->[method]();
 *
 * $registry['PSX/Loader']->[method]();  // alternative
 *
 *
 * // or we can use normal name. Note if you load an object called foo_loader
 * // after this you will get the foo_loader object
 * $registry->get('Loader')->[method]();
 *
 * $registry['Loader']->[method]();  // alternative
 * </code>
 *
 * To get the complete number of loaded classes in the library you can use the
 * count() function on the registry object i.e.
 * <code>
 * if(count($this->registry) > 8)
 * {
 * 	// you have loaded more then 8 objects
 * }
 * </code>
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Registry
 * @version    $Revision: 260 $
 */
class PSX_Registry implements Countable, ArrayAccess
{
	/**
	 * The array that holds all objects
	 *
	 * @var array
	 */
	private $list;

	/**
	 * The list of keys so that you can access a class without the fully
	 * qualified name
	 *
	 * @var array
	 */
	public $key;

	/**
	 * get
	 *
	 * If the path is valid it returns an object else it throws an exception
	 *
	 * @param string $path
	 * @return object
	 */
	public function get($path)
	{
		if($this->has($path))
		{
			if(isset($this->list[$path]))
			{
				return $this->list[$path];
			}
			else
			{
				return $this->list[$this->key[$path]];
			}
		}
		else
		{
			throw new PSX_Registry_Exception('Object ' . $path . ' doesnt exist');
		}
	}

	/**
	 * set
	 *
	 * Add an object to the registry.
	 *
	 * @param object $handle
	 * @return void
	 */
	public function set($handle)
	{
		if(is_object($handle))
		{
			$path = str_replace(array('_', '\\'), '/', get_class($handle));
		}
		else
		{
			throw new PSX_Registry_Exception('You can register only objects');
		}

		if(!$this->has($path, true))
		{
			$this->list[$path] = $handle;

			$pos = strrpos($path, '/');
			$key = $pos !== false ? substr($path, $pos + 1) : $path;

			$this->key[$key] = $path;
		}
		else
		{
			throw new PSX_Registry_Exception('Object ' . $path . ' already set');
		}
	}

	/**
	 * has
	 *
	 * Checks whether the full qualified $path exists
	 *
	 * @param string $path
	 * @return void
	 */
	public function has($path, $absolute = false)
	{
		if($absolute === true)
		{
			return isset($this->list[$path]);
		}
		else
		{
			return isset($this->list[$path]) || isset($this->key[$path]);
		}
	}

	/**
	 * get_container
	 *
	 * Returns the underlying array that contains the objects
	 *
	 * @return array
	 */
	public function getContainer()
	{
		return $this->list;
	}

	/**
	 * clear
	 *
	 * Delete all entries from the container
	 *
	 * @return void
	 */
	public function clear()
	{
		$this->list = array();
	}

	// Countable
	public function count()
	{
		return count($this->list);
	}

	// ArrayAccess
	public function offsetSet($offset, $value)
	{
		$this->set($value);
	}

	public function offsetExists($offset)
	{
		return $this->has($offset);
	}

	public function offsetUnset($offset)
	{
	}

	public function offsetGet($offset)
	{
		return $this->get($offset);
	}
}

