<?php
/*
 *  $Id: Sql.php 275 2011-05-01 11:26:41Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Sql
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Sql
 * @version    $Revision: 275 $
 */
class PSX_Sql
{
	const LOW_PRIORITY  = 0x1;
	const DELAYED       = 0x2;
	const HIGH_PRIORITY = 0x4;
	const QUICK         = 0x8;
	const IGNORE        = 0x16;

	const FETCH_ASSOC   = 0x0;
	const FETCH_OBJECT  = 0x1;

	const SELECT_ALL    = 0x0;
	const SELECT_ROW    = 0x1;
	const SELECT_COL    = 0x2;
	const SELECT_FIELD  = 0x3;

	const SORT_ASC      = 0x0;
	const SORT_DESC     = 0x1;

	private $driver;
	private $count = 0;
	private $list  = array();

	public function __construct(PSX_Sql_DriverInterface $driver, $host, $user, $pw, $db)
	{
		$this->driver = $driver;

		if(!($this->driver->connect($host, $user, $pw, $db)))
		{
			throw new PSX_Sql_Exception('Couldnt connect to database!');
		}
		else
		{
			// set default charset
			$this->exec('SET NAMES "utf8"');
		}
	}

	/**
	 * assoc
	 *
	 * Main method for data selection. It returns either an associative
	 * array with the data or false. It uses prepared statments so you can
	 * write questionmarks in your query and provide the values in the
	 * $params array
	 *
	 * @param string $sql
	 * @param array $params
	 * @return array|false
	 */
	public function assoc($sql, array $params = array())
	{
		$stmt = $this->prepare($sql);

		if(count($params) > 0)
		{
			foreach($params as $v)
			{
				$stmt->bindParam($v);
			}
		}

		$stmt->execute();


		$last_error = $stmt->error();

		if(!empty($last_error))
		{
			throw new PSX_Sql_Exception($last_error);
		}


		$this->count++;

		$content = false;

		if($stmt->numRows() > 0)
		{
			$content = $stmt->fetchAssoc();
		}

		return $content;
	}

	/**
	 * object
	 *
	 * Method for data selection. This is an ORM like method where you get
	 * an array of instances of the $class. As third argument you can give
	 * an array og variables wich are passed to the constructor of the
	 * $class. This method is maybe slower then assoc because it creates for
	 * each record an instance of $class
	 *
	 * @param string $sql
	 * @param string $class
	 * @param array $args
	 * @param array $params
	 * @return array|false
	 */
	public function object($sql, array $params = array(), $class = 'stdClass', array $args = array())
	{
		$stmt = $this->prepare($sql);

		if(count($params) > 0)
		{
			foreach($params as $v)
			{
				$stmt->bindParam($v);
			}
		}

		$stmt->execute();


		$last_error = $stmt->error();

		if(!empty($last_error))
		{
			throw new PSX_Sql_Exception($last_error);
		}


		$this->count++;

		$content = false;

		if($stmt->numRows() > 0)
		{
			$content = $stmt->fetchObject($class, $args);
		}

		return $content;
	}

	/**
	 * query
	 *
	 * Main method for data manipulation (INSERT, UPDATE, REPLACE, DELETE).
	 * It returns the number of affected rows. It uses prepared statments so
	 * you can write questionmarks in your query and provide the values in
	 * the $params array
	 *
	 * @param string $sql
	 * @param array $params
	 * @return boolean
	 */
	public function query($sql, array $params = array())
	{
		$stmt = $this->prepare($sql);

		if(count($params) > 0)
		{
			foreach($params as $v)
			{
				$stmt->bindParam($v);
			}
		}

		$stmt->execute();


		$last_error = $stmt->error();

		if(!empty($last_error))
		{
			throw new PSX_Sql_Exception($last_error);
		}


		$this->count++;

		return $stmt->numRows();
	}

	/**
	 * exec
	 *
	 * Main method for executing queries where you dont need prepared
	 * statments i.e. setting the charset or selecting another database.
	 *
	 * @param string $sql
	 * @return boolean
	 */
	public function exec($sql)
	{
		if($this->driver->exec($sql) === false)
		{
			throw new PSX_Sql_Exception($this->driver->error());
		}
		else
		{
			$this->count++;

			return true;
		}
	}

	/**
	 * getAll
	 *
	 * @param string $sql
	 * @param array $params
	 * @return array
	 */
	 public function getAll($sql, array $params = array(), $mode = 0, $class = 'stdClass', array $args = array())
	{
		$result = $this->getResult($sql, $params, $mode, $class, $args);

		if(!empty($result))
		{
			return $result;
		}

		return array();
	}

	/**
	 * getRow
	 *
	 * @param string $sql
	 * @param array $params
	 * @return array
	 */
	public function getRow($sql, array $params = array(), $mode = 0, $class = 'stdClass', array $args = array())
	{
		$content = array();
		$result  = $this->getResult($sql, $params, $mode, $class, $args);

		if(!empty($result))
		{
			$content = current($result);

			unset($result);
		}

		return $content;
	}

	/**
	 * getCol
	 *
	 * @param string $sql
	 * @param array $params
	 * @return array
	 */
	public function getCol($sql, array $params = array())
	{
		$content = array();
		$result  = $this->getResult($sql, $params, self::FETCH_ASSOC);

		if(!empty($result))
		{
			foreach($result as $row)
			{
				$content[] = current($row);
			}

			unset($result);
		}

		return $content;
	}

	/**
	 * getField
	 *
	 * @param string $sql
	 * @param array $params
	 * @return array
	 */
	public function getField($sql, array $params = array())
	{
		$content = false;
		$result  = $this->getResult($sql, $params, self::FETCH_ASSOC);

		if(!empty($result))
		{
			$row = current($result);

			unset($result);

			$content = current($row);
		}

		return $content;
	}

	public function getResult($sql, array $params = array(), $mode = 0, $class = 'stdClass', array $args = array())
	{
		$result = null;

		switch($mode)
		{
			case self::FETCH_ASSOC:

				$result = $this->assoc($sql, $params);

				break;

			case self::FETCH_OBJECT:

				$result = $this->object($sql, $params, $class, $args);

				break;

			default:

				throw new PSX_Sql_Exception('Invalid mode');
		}

		return $result;
	}

	/**
	 * select
	 *
	 * @param string $table
	 * @param array $fields
	 * @param PSX_Sql_Condition $condition
	 * @return array
	 */
	public function select($table, array $fields, PSX_Sql_Condition $condition = null, $select = 0, $sort_by = null, $sort_order = 0)
	{
		if(!empty($fields))
		{
			if($condition !== null)
			{
				$sql    = 'SELECT ' . implode(', ', array_map(__CLASS__ . '::helpQuote', $fields)) . ' FROM `' . $table . '` ' . $condition->getStatment() . ' ';
				$params = $condition->getValues();
			}
			else
			{
				$sql    = 'SELECT ' . implode(', ', array_map(__CLASS__ . '::helpQuote', $fields)) . ' FROM `' . $table . '` ';
				$params = array();
			}

			if($sort_by !== null)
			{
				$sql.= 'ORDER BY `' . $sort_by . '` ' . ($sort_order == self::SORT_ASC ? 'ASC' : 'DESC');
			}

			$result = null;

			switch($select)
			{
				case self::SELECT_ALL:

					$result = $this->getAll($sql, $params);

					break;

				case self::SELECT_ROW:

					$result = $this->getRow($sql, $params);

					break;

				case self::SELECT_COL:

					$result = $this->getCol($sql, $params);

					break;

				case self::SELECT_FIELD:

					$result = $this->getField($sql, $params);

					break;
			}

			return $result;
		}
		else
		{
			throw new PSX_Sql_Exception('Array must not be empty');
		}
	}

	/**
	 * insert
	 *
	 * @param string $table
	 * @param array $params
	 * @param int $modifier
	 * @return boolean
	 */
	public function insert($table, array $params, $modifier = 0)
	{
		if(!empty($params))
		{
			$keywords = '';

			if($modifier & self::LOW_PRIORITY)
			{
				$keywords.= ' LOW_PRIORITY ';
			}
			elseif($modifier & self::DELAYED)
			{
				$keywords.= ' DELAYED ';
			}
			elseif($modifier & self::HIGH_PRIORITY)
			{
				$keywords.= ' HIGH_PRIORITY ';
			}

			if($modifier & self::IGNORE)
			{
				$keywords.= ' IGNORE ';
			}

			$keys = array_keys($params);
			$sql  = 'INSERT ' . $keywords . ' `' . $table . '` SET ' . implode(', ', array_map(__CLASS__ . '::helpPrepare', $keys));

			return $this->query($sql, $params);
		}
		else
		{
			throw new PSX_Sql_Exception('Array must not be empty');
		}
	}

	/**
	 * update
	 *
	 * @param string $table
	 * @param array $params
	 * @param PSX_Sql_Condition $condition
	 * @param int $modifier
	 * @return int
	 */
	public function update($table, array $params, PSX_Sql_Condition $condition = null, $modifier = 0)
	{
		if(!empty($params))
		{
			$keywords = '';

			if($modifier & self::LOW_PRIORITY)
			{
				$keywords.= ' LOW_PRIORITY ';
			}

			if($modifier & self::IGNORE)
			{
				$keywords.= ' IGNORE ';
			}

			$keys = array_keys($params);

			if($condition !== null)
			{
				$sql    = 'UPDATE ' . $keywords . ' `' . $table . '` SET ' . implode(', ', array_map(__CLASS__ . '::helpPrepare', $keys)) . ' ' . $condition->getStatment();
				$params = array_merge(array_values($params), $condition->getValues());
			}
			else
			{
				$sql    = 'UPDATE ' . $keywords . ' `' . $table . '` SET ' . implode(', ', array_map(__CLASS__ . '::helpPrepare', $keys));
				$params = array_values($params);
			}

			return $this->query($sql, $params);
		}
		else
		{
			throw new PSX_Sql_Exception('Array must not be empty');
		}
	}

	/**
	 * replace
	 *
	 * @param string $table
	 * @param array $params
	 * @param PSX_Sql_Condition $condition
	 * @param int $modifier
	 * @return int
	 */
	public function replace($table, array $params, PSX_Sql_Condition $condition = null, $modifier = 0)
	{
		if(!empty($params))
		{
			$keywords = '';

			if($modifier & self::LOW_PRIORITY)
			{
				$keywords.= ' LOW_PRIORITY ';
			}
			else if($modifier & self::DELAYED)
			{
				$keywords.= ' DELAYED ';
			}

			$keys = array_keys($params);

			if($condition !== null)
			{
				$sql    = 'REPLACE ' . $keywords . ' `' . $table . '` SET ' . implode(', ', array_map(__CLASS__ . '::helpPrepare', $keys)) . ' ' . $condition->getStatment();
				$params = array_merge(array_values($params), $condition->getValues());
			}
			else
			{
				$sql    = 'REPLACE ' . $keywords . ' `' . $table . '` SET ' . implode(', ', array_map(__CLASS__ . '::helpPrepare', $keys));
				$params = array_values($params);
			}

			return $this->query($sql, $params);
		}
		else
		{
			throw new PSX_Sql_Exception('Array must not be empty');
		}
	}

	/**
	 * delete
	 *
	 * @param string $table
	 * @param PSX_Sql_Condition $condition
	 * @param int $modifier
	 * @return int
	 */
	public function delete($table, PSX_Sql_Condition $condition = null, $modifier = 0)
	{
		$keywords = '';

		if($modifier & self::LOW_PRIORITY)
		{
			$keywords.= ' LOW_PRIORITY ';
		}

		if($modifier & self::QUICK)
		{
			$keywords.= ' QUICK ';
		}

		if($modifier & self::IGNORE)
		{
			$keywords.= ' IGNORE ';
		}

		if($condition !== null)
		{
			$sql    = 'DELETE ' . $keywords . ' FROM `' . $table . '` ' . $condition->getStatment();
			$params = $condition->getValues();
		}
		else
		{
			$sql    = 'DELETE ' . $keywords . ' FROM `' . $table . '`';
			$params = array();
		}

		return $this->query($sql, $params);
	}

	public function count($table, PSX_Sql_Condition $condition = null)
	{
		if($condition !== null)
		{
			$sql    = 'SELECT COUNT(*) FROM `' . $table . '` ' . $condition->getStatment();
			$params = $condition->getValues();
		}
		else
		{
			$sql    = 'SELECT COUNT(*) FROM `' . $table . '`';
			$params = array();
		}

		return $this->getField($sql, $params);
	}

	public function beginTransaction()
	{
		$this->driver->beginTransaction();
	}

	public function commit()
	{
		$this->driver->commit();
	}

	public function rollback()
	{
		$this->driver->rollback();
	}

	public function close()
	{
		$this->driver->close();
	}

	public function getLastInsertId()
	{
		return $this->driver->lastInsertId();
	}

	public function quote($string)
	{
		return $this->driver->quote($string);
	}

	public function getCount()
	{
		return $this->count;
	}

	public function getDriver()
	{
		return $this->driver;
	}

	public function prepare($sql)
	{
		$key = md5($sql);

		if(!isset($this->list[$key]))
		{
			$stmt = $this->driver->prepare($sql);

			$this->list[$key] = $stmt;
		}
		else
		{
			$stmt = $this->list[$key];
		}

		return $stmt;
	}

	public static function helpQuote($str)
	{
		return '`' . $str . '`';
	}

	public static function helpPrepare($str)
	{
		return '`' . $str . '` = ?';
	}
}


