<?php
/*
 *  $Id: Validate.php 250 2011-03-25 19:17:41Z k42b3.x $
 *
 * psx
 * A object oriented and modular based PHP framework for developing
 * dynamic web applications. For the current version and informations
 * visit <http://phpsx.org>
 *
 * Copyright (c) 2011 Christoph Kappestein <k42b3.x@gmail.com>
 *
 * This file is part of psx. psx is free software: you can
 * redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or any later version.
 *
 * psx is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with psx. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * PSX_Validate
 *
 * This class offers methods to sanitize values that came from untrusted sources
 * The apply methods is the heart of this class here an example how you can
 * easily validate data.
 * <code>
 * $val = new PSX_Validate();
 *
 * $value = $val->apply($input, 'string', array(new PSX_Filter_Validate_Length(3, 8)));
 *
 * if($value === false)
 * {
 * 	// validation fails
 * }
 * else
 * {
 * 	// the variable $value is an string with min 3 and max 8 chars. The
 * 	// method validate cast the variable to a string
 * }
 * </code>
 *
 * The method validate returns the value if the validation was successful or
 * false if it fails. You can cast the variable to string, integer, float or
 * boolean. Optional you can pass an array of filters wich validates the value.
 * Here some examples:
 * <code>
 * $val = new PSX_Validate();
 *
 * // we use the length and email filter
 * $value = $val->apply($input, 'string', array(new PSX_Filter_Length(3, 32), new PSX_Filter_Email()));
 *
 * // we use the regular expression filter. The value must contain the string
 * // "php"
 * $value = $val->apply($input, 'string', array(new PSX_Filter_Regexp('/php/')));
 *
 * // we us the length filter and the value must be an interger wich is min 10
 * // and max 100
 * $value = $val->apply($input, 'integer', array(new PSX_Filter_Length(10, 100)));
 * </code>
 *
 * If you have validate the values you can check the filter whether their was an
 * error.
 * <code>
 * if($filter->hasError())
 * {
 * 	echo 'The following errors occured:' . "\n";
 * 	echo implode("\n", $filter->getError());
 * }
 * else
 * {
 * 	echo 'No validation error occured';
 * }
 * </code>
 *
 * @author     Christoph Kappestein <k42b3.x@gmail.com>
 * @license    http://www.gnu.org/licenses/gpl.html GPLv3
 * @link       http://phpsx.org
 * @category   PSX
 * @package    PSX_Validate
 * @version    $Revision: 250 $
 */
class PSX_Validate
{
	private $error = array();

	/**
	 * apply
	 *
	 * @param string $value
	 * @param string $type
	 * @param array $filter
	 * @return false|integer|float|string|boolean
	 */
	public function apply($value, $type = 'string', array $filter = array(), $key = '', $title = '', $returnValue = false)
	{
		if($value === false)
		{
			$this->addError($key, 'Value not set');

			return $returnValue;
		}

		switch($type)
		{
			case 'int':
			case 'integer':

				$value = (integer) $value;

				break;

			case 'str':
			case 'string':

				$value = (string)  $value;

				break;

			case 'double':
			case 'float':

				$value = (float)   $value;

				break;

			case 'bool':
			case 'boolean':

				$value = (boolean) $value;

				break;

			default:

				throw new PSX_Validate_Exception('Invalid data type');

				break;
		}

		foreach($filter as $f)
		{
			$return = $f->apply($value);

			if($return === false)
			{
				$this->addError($key, sprintf($f->getErrorMsg(), $title));

				return $returnValue;
			}
			else if($return === true)
			{
				// the filter returns true so the validation was successful
			}
			else
			{
				$value = $return;
			}
		}

		return $value;
	}

	public function addError($key, $msg)
	{
		if(!empty($this->callback))
		{
			$func = $this->callback;

			$func($msg);
		}

		$this->error[$key] = $msg;
	}

	public function hasError()
	{
		return count($this->error) > 0;
	}

	public function getError()
	{
		return $this->error;
	}

	public function getLastError()
	{
		return end($this->error);
	}
}
